indexing

   description: "Class that describes socket addresses, i.e. host name%
   %and port number combination. Or in C terms struct sockaddr_in."

   author: "Berend de Boer"
   date: "$Date: 2001/12/27 $"
   revision: "$Revision: #1 $"


class

   SUS_SOCKET_ADDRESS


inherit

   SUS_BASE

   SAPI_IN


creation

   make


feature -- creation

   make (a_host: SUS_HOST; a_service: SUS_SERVICE) is
      require
         valid_host: a_host /= Void
         valid_service: a_service /= Void
      do
         host := a_host
         service := a_service
         -- ip4 only for now
         create buf.allocate_and_clear (posix_sockaddr_in_size)
         set_address (host.addresses.lower)
      ensure
         host_set: host = a_host
         service_set: service = a_service
      end


feature -- Eiffel features

   host: SUS_HOST

   service: SUS_SERVICE


feature -- fill socket structure, so `ptr' returns something valid

   set_address (item: INTEGER) is
         -- use one of the ip addresses of `host' as the socket address
      require
         existing_ip_address_item: item >= host.addresses.lower and
                                   item <= host.addresses.upper
      local
         ip_address: SUS_IP_ADDRESS
      do
         posix_set_sockaddr_in_sin_family (buf.ptr, host.address_type)
         posix_set_sockaddr_in_sin_port (buf.ptr, posix_htons (service.port))
         ip_address := host.addresses.item (item)
         posix_set_sockaddr_in_sin_addr (buf.ptr, ip_address.ptr)
      end


feature -- features the C API calls like

   length: INTEGER is
         -- size of my struct sockaddr_in
      do
         Result := buf.capacity
      ensure
         valid_length: length > 0 -- >= 16 usually
      end

   ptr: POINTER is
         -- points to struct sockaddr_in or sockaddr_in6
      do
         Result := buf.ptr
      ensure
         valid_result: Result /= default_pointer
      end


feature {NONE} -- private state

   buf: STDC_BUFFER
         -- pointer to a struct sockaddr_in structure


invariant

   has_host: host /= Void
   has_service: service /= Void
   valid_buf: buf /= Void and then buf.capacity >= length

end
