#include "w_windows.h"

/* win32 support */

/* following function idea thanks to the Apache team */

/* Number of 100 nano=seconds between the beginning of the Windows epoch
 * (Jan. 1, 1601) and the Unix epoch (Jan. 1, 1970)
 */
#define POSIX_DELTA_EPOCH   116444736000000000

time_t FileTimeToUnixTime(FILETIME input)
{
  LONGLONG t;
  if ((input.dwHighDateTime == 0) && (input.dwLowDateTime == 0))
    { t = 0; }
  else
  {
    /* Convert FILETIME one 64 bit number so we can work with it. */
    t = input.dwHighDateTime;
    t = t << 32;
    t |= input.dwLowDateTime;
    t -= POSIX_DELTA_EPOCH;  /* Convert from Windows epoch to Unix epoch */
    t /= 10000000;    /* Convert from 100 nano-sec periods to seconds. */
  }
  return t;
}


/* error handling */

EIF_INTEGER posix_getlasterror()
{
  return GetLastError();
}

EIF_POINTER posix_geterrormessage(EIF_INTEGER error_number)
{
  LPVOID lpvMessageBuffer;

  FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER|FORMAT_MESSAGE_FROM_SYSTEM,
                NULL, error_number,
                MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
                (LPTSTR)&lpvMessageBuffer, 0, NULL);
  return (lpvMessageBuffer);
}


/* error constants */

EIF_INTEGER const_error_broken_pipe()
{
  return ERROR_BROKEN_PIPE;
}

EIF_INTEGER const_error_invalid_handle()
{
  return ERROR_INVALID_HANDLE;
}

EIF_INTEGER const_error_no_data()
{
  return ERROR_NO_DATA;
}

EIF_INTEGER const_error_not_supported()
{
  return ERROR_NOT_SUPPORTED;
}


/* find functions */

EIF_POINTER posix_findfirstfile(EIF_POINTER filename, EIF_POINTER find_file_data)
{
  return ( FindFirstFile(filename, find_file_data));
}

EIF_BOOLEAN posix_findnextfile(EIF_POINTER handle, EIF_POINTER find_file_data)
{
  return ( FindNextFile (handle, find_file_data));
}

EIF_BOOLEAN posix_findclose(EIF_POINTER handle)
{
  return (FindClose (handle));
}


/* directories */

EIF_INTEGER posix_gettemppath (EIF_INTEGER len, EIF_POINTER buf)
{
  return GetTempPath (len, buf);
}


/* basic handle stuff */

EIF_BOOLEAN posix_closehandle (EIF_INTEGER hObject)
{
  return CloseHandle( (HANDLE) hObject);
}

EIF_BOOLEAN posix_duplicatehandle (EIF_INTEGER hSourceProcessHandle, EIF_INTEGER hSourceHandle, EIF_INTEGER hTargetProcessHandle, EIF_POINTER lpTargetHandle, EIF_INTEGER dwDesiredAccess, EIF_BOOLEAN bInheritHandle, EIF_INTEGER dwOptions)
{
  return DuplicateHandle( (HANDLE) hSourceProcessHandle, (HANDLE) hSourceHandle, (HANDLE) hTargetProcessHandle, lpTargetHandle, dwDesiredAccess, bInheritHandle, dwOptions);
}

EIF_INTEGER posix_waitforsingleobject(EIF_INTEGER hHandle, EIF_INTEGER dwMilliseconds)
{
  return WaitForSingleObject( (HANDLE) hHandle, dwMilliseconds);
}


/* file descriptors handles */

EIF_INTEGER posix_createfile(EIF_POINTER lpFileName, EIF_INTEGER dwDesiredAccess, EIF_INTEGER dwShareMode, EIF_POINTER lpSecurityAttributes, EIF_INTEGER dwCreationDistribution, EIF_INTEGER dwFlagsAndAttributes, EIF_INTEGER hTemplateFile)
{
  return (EIF_INTEGER) CreateFile(lpFileName, dwDesiredAccess, dwShareMode, lpSecurityAttributes, dwCreationDistribution, dwFlagsAndAttributes, (HANDLE) hTemplateFile);
}

EIF_BOOLEAN posix_createpipe(EIF_INTEGER hReadPipe, EIF_INTEGER hWritePipe, EIF_POINTER lpPipeAttributes, EIF_INTEGER nSize)
{
  return CreatePipe((void *) hReadPipe, (void *) hWritePipe, lpPipeAttributes, nSize);
}

EIF_BOOLEAN posix_getfileinformationbyhandle(EIF_INTEGER hFile, EIF_POINTER lpFileInformation)
{
  return GetFileInformationByHandle((HANDLE) hFile, lpFileInformation);
}

EIF_INTEGER posix_getfiletype(EIF_INTEGER hFile)
{
  return GetFileType((HANDLE) hFile);
}

EIF_INTEGER posix_getstdhandle (EIF_INTEGER nStdHandle)
{
  return (EIF_INTEGER) GetStdHandle (nStdHandle);
}

EIF_BOOLEAN posix_readfile(EIF_INTEGER hFile, EIF_POINTER lpBuffer, EIF_INTEGER nNumberOfBytesToRead, EIF_POINTER lpNumberOfBytesRead, EIF_POINTER lpOverlapped)
{
  return ReadFile((HANDLE) hFile, lpBuffer, nNumberOfBytesToRead, lpNumberOfBytesRead, lpOverlapped);
}

EIF_INTEGER posix_setfilepointer (EIF_INTEGER hFile, EIF_INTEGER lDistanceToMove, EIF_POINTER lpDistanceToMoveHigh, EIF_INTEGER dwMoveMethod)
{
  return SetFilePointer((HANDLE) hFile, lDistanceToMove, lpDistanceToMoveHigh, dwMoveMethod);
}

EIF_BOOLEAN posix_setstdhandle (EIF_INTEGER nStdHandle, EIF_INTEGER hHandle)
{
  return SetStdHandle(nStdHandle, (HANDLE) hHandle);
}

EIF_BOOLEAN posix_writefile(EIF_INTEGER hFile, EIF_POINTER lpBuffer, EIF_INTEGER nNumberOfBytesToWrite, EIF_POINTER lpNumberOfBytesWritten, EIF_POINTER lpOverlapped)
{
  return WriteFile((HANDLE) hFile, lpBuffer, nNumberOfBytesToWrite, lpNumberOfBytesWritten, lpOverlapped);

}


/* process */

EIF_BOOLEAN posix_createprocess(EIF_POINTER lpApplicationName, EIF_POINTER lpCommandLine, EIF_POINTER lpProcessAttributes, EIF_POINTER lpThreadAttributes, EIF_BOOLEAN bInheritHandles, EIF_INTEGER dwCreationFlags, EIF_POINTER lpEnvironment, EIF_POINTER lpCurrentDirectory, EIF_POINTER lpStartupInfo, EIF_POINTER lpProcessInformation)
{
  return CreateProcess(lpApplicationName, lpCommandLine, lpProcessAttributes, lpThreadAttributes, bInheritHandles, dwCreationFlags, lpEnvironment, lpCurrentDirectory, lpStartupInfo, lpProcessInformation);
}

EIF_INTEGER posix_getcurrentprocess()
{
  return (EIF_INTEGER) GetCurrentProcess();
}

EIF_INTEGER posix_getcurrentprocessid()
{
  return GetCurrentProcessId();
}

EIF_BOOLEAN posix_getexitcodeprocess(EIF_INTEGER hProcess, EIF_POINTER lpExitCode)
{
  return GetExitCodeProcess ( (HANDLE) hProcess, lpExitCode);
}

EIF_BOOLEAN posix_terminateprocess(EIF_INTEGER hProcess, EIF_INTEGER uExitCode)
{
  return TerminateProcess((void *)hProcess, uExitCode);
}


/* various */

void posix_sleep(EIF_INTEGER dwMilliseconds)
{
  Sleep(dwMilliseconds);
}


/* struct WIN32_FIND_DATA  */

EIF_INTEGER posix_win32_find_data_size()
{
  return (sizeof (WIN32_FIND_DATA));
}

EIF_POINTER posix_win32_find_data_filename (PWIN32_FIND_DATA find_data)
{
  return (find_data->cFileName);
}


/* struct SECURITY_ATTRIBUTES */

EIF_INTEGER posix_security_attributes_size()
{
   return (sizeof (struct _SECURITY_ATTRIBUTES));
}

EIF_INTEGER posix_security_attributes_nlength(struct _SECURITY_ATTRIBUTES *p)
{
  return p->nLength;
}

EIF_POINTER posix_security_attributes_lpsecuritydescriptor(struct _SECURITY_ATTRIBUTES *p)
{
  return p->lpSecurityDescriptor;
}

EIF_BOOLEAN posix_security_attributes_binherithandle(struct _SECURITY_ATTRIBUTES *p)
{
  return p->bInheritHandle;
}


void posix_set_security_attributes_nlength(struct _SECURITY_ATTRIBUTES *p, EIF_INTEGER nLength)
{
  p->nLength = nLength;
}

void posix_set_security_attributes_lpsecuritydescriptor(struct _SECURITY_ATTRIBUTES *p, EIF_POINTER lpSecurityDescriptor)
{
  p->lpSecurityDescriptor = lpSecurityDescriptor;
}

void posix_set_security_attributes_binherithandle(struct _SECURITY_ATTRIBUTES *p, EIF_BOOLEAN bInheritHandle)
{
  p->bInheritHandle = bInheritHandle;
}


/* struct STARTUPINFOA */

EIF_INTEGER posix_startupinfo_size()
{
   return (sizeof (struct _STARTUPINFOA));
}

EIF_INTEGER posix_startupinfo_cb(struct _STARTUPINFOA *p)
{
  return p->cb;
}

EIF_POINTER posix_startupinfo_lpreserved(struct _STARTUPINFOA *p)
{
  return p->lpReserved;
}

EIF_POINTER posix_startupinfo_lpdesktop(struct _STARTUPINFOA *p)
{
  return p->lpDesktop;
}

EIF_POINTER posix_startupinfo_lptitle(struct _STARTUPINFOA *p)
{
  return p->lpTitle;
}

EIF_INTEGER posix_startupinfo_dwx(struct _STARTUPINFOA *p)
{
  return p->dwX;
}

EIF_INTEGER posix_startupinfo_dwy(struct _STARTUPINFOA *p)
{
  return p->dwY;
}

EIF_INTEGER posix_startupinfo_dwxsize(struct _STARTUPINFOA *p)
{
  return p->dwXSize;
}

EIF_INTEGER posix_startupinfo_dwysize(struct _STARTUPINFOA *p)
{
  return p->dwYSize;
}

EIF_INTEGER posix_startupinfo_dwxcountchars(struct _STARTUPINFOA *p)
{
  return p->dwXCountChars;
}

EIF_INTEGER posix_startupinfo_dwycountchars(struct _STARTUPINFOA *p)
{
  return p->dwYCountChars;
}

EIF_INTEGER posix_startupinfo_dwfillattribute(struct _STARTUPINFOA *p)
{
  return p->dwFillAttribute;
}

EIF_INTEGER posix_startupinfo_dwflags(struct _STARTUPINFOA *p)
{
  return p->dwFlags;
}

EIF_INTEGER posix_startupinfo_wshowwindow(struct _STARTUPINFOA *p)
{
  return p->wShowWindow;
}

EIF_INTEGER posix_startupinfo_cbreserved2(struct _STARTUPINFOA *p)
{
  return p->cbReserved2;
}

EIF_POINTER posix_startupinfo_lpreserved2(struct _STARTUPINFOA *p)
{
  return p->lpReserved2;
}

EIF_INTEGER posix_startupinfo_hstdinput(struct _STARTUPINFOA *p)
{
  return (EIF_INTEGER) p->hStdInput;
}

EIF_INTEGER posix_startupinfo_hstdoutput(struct _STARTUPINFOA *p)
{
  return (EIF_INTEGER) p->hStdOutput;
}

EIF_INTEGER posix_startupinfo_hstderror(struct _STARTUPINFOA *p)
{
  return (EIF_INTEGER) p->hStdError;
}


void posix_set_startupinfo_cb(struct _STARTUPINFOA *p, EIF_INTEGER cb)
{
  p->cb = cb;
}

void posix_set_startupinfo_lpreserved(struct _STARTUPINFOA *p, EIF_POINTER lpReserved)
{
  p->lpReserved = lpReserved;
}

void posix_set_startupinfo_lpdesktop(struct _STARTUPINFOA *p, EIF_POINTER lpDesktop)
{
  p->lpDesktop = lpDesktop;
}

void posix_set_startupinfo_lptitle(struct _STARTUPINFOA *p, EIF_POINTER lpTitle)
{
  p->lpTitle = lpTitle;
}

void posix_set_startupinfo_dwx(struct _STARTUPINFOA *p, EIF_INTEGER dwX)
{
  p->dwX = dwX;
}

void posix_set_startupinfo_dwy(struct _STARTUPINFOA *p, EIF_INTEGER dwY)
{
  p->dwY = dwY;
}

void posix_set_startupinfo_dwxsize(struct _STARTUPINFOA *p, EIF_INTEGER dwXSize)
{
  p->dwXSize = dwXSize;
}

void posix_set_startupinfo_dwysize(struct _STARTUPINFOA *p, EIF_INTEGER dwYSize)
{
  p->dwYSize = dwYSize;
}

void posix_set_startupinfo_dwxcountchars(struct _STARTUPINFOA *p, EIF_INTEGER dwXCountChars)
{
  p->dwXCountChars = dwXCountChars;
}

void posix_set_startupinfo_dwycountchars(struct _STARTUPINFOA *p, EIF_INTEGER dwYCountChars)
{
  p->dwYCountChars = dwYCountChars;
}

void posix_set_startupinfo_dwfillattribute(struct _STARTUPINFOA *p, EIF_INTEGER dwFillAttribute)
{
  p->dwFillAttribute = dwFillAttribute;
}

void posix_set_startupinfo_dwflags(struct _STARTUPINFOA *p, EIF_INTEGER dwFlags)
{
  p->dwFlags = dwFlags;
}

void posix_set_startupinfo_wshowwindow(struct _STARTUPINFOA *p, EIF_INTEGER wShowWindow)
{
  p->wShowWindow = wShowWindow;
}

void posix_set_startupinfo_cbreserved2(struct _STARTUPINFOA *p, EIF_INTEGER cbReserved2)
{
  p->cbReserved2 = cbReserved2;
}

void posix_set_startupinfo_lpreserved2(struct _STARTUPINFOA *p, EIF_POINTER lpReserved2)
{
  p->lpReserved2 = lpReserved2;
}

void posix_set_startupinfo_hstdinput(struct _STARTUPINFOA *p, EIF_INTEGER hStdInput)
{
  p->hStdInput = (HANDLE) hStdInput;
}

void posix_set_startupinfo_hstdoutput(struct _STARTUPINFOA *p, EIF_INTEGER hStdOutput)
{
  p->hStdOutput = (HANDLE) hStdOutput;
}

void posix_set_startupinfo_hstderror(struct _STARTUPINFOA *p, EIF_INTEGER hStdError)
{
  p->hStdError = (HANDLE) hStdError;
}


/* struct _PROCESS_INFORMATION */

EIF_INTEGER posix_process_information_size()
{
   return (sizeof (struct _PROCESS_INFORMATION));
}

EIF_INTEGER posix_process_information_hprocess(struct _PROCESS_INFORMATION *p)
{
  return (EIF_INTEGER) p->hProcess;
}

EIF_INTEGER posix_process_information_hthread(struct _PROCESS_INFORMATION *p)
{
  return (EIF_INTEGER) p->hThread;
}

EIF_INTEGER posix_process_information_dwprocessid(struct _PROCESS_INFORMATION *p)
{
  return p->dwProcessId;
}

EIF_INTEGER posix_process_information_dwthreadid(struct _PROCESS_INFORMATION *p)
{
  return p->dwThreadId;
}


/* struct BY_HANDLE_FILE_INFORMATION */

EIF_INTEGER posix_by_handle_file_information_size()
{
   return (sizeof (struct _BY_HANDLE_FILE_INFORMATION));
}

EIF_INTEGER posix_by_handle_file_information_dwfileattributes(struct _BY_HANDLE_FILE_INFORMATION *p)
{
  return p->dwFileAttributes;
}

EIF_INTEGER posix_by_handle_file_information_ftcreationtime(struct _BY_HANDLE_FILE_INFORMATION *p)
{
  return FileTimeToUnixTime(p->ftCreationTime);
}

EIF_INTEGER posix_by_handle_file_information_ftlastaccesstime(struct _BY_HANDLE_FILE_INFORMATION *p)
{
  return FileTimeToUnixTime(p->ftLastAccessTime);
}

EIF_INTEGER posix_by_handle_file_information_ftlastwritetime(struct _BY_HANDLE_FILE_INFORMATION *p)
{
  return FileTimeToUnixTime(p->ftLastWriteTime);
}

EIF_INTEGER posix_by_handle_file_information_dwvolumeserialnumber(struct _BY_HANDLE_FILE_INFORMATION *p)
{
  return p->dwVolumeSerialNumber;
}

EIF_INTEGER posix_by_handle_file_information_nfilesizehigh(struct _BY_HANDLE_FILE_INFORMATION *p)
{
  return p->nFileSizeHigh;
}

EIF_INTEGER posix_by_handle_file_information_nfilesizelow(struct _BY_HANDLE_FILE_INFORMATION *p)
{
  return p->nFileSizeLow;
}

EIF_INTEGER posix_by_handle_file_information_nnumberoflinks(struct _BY_HANDLE_FILE_INFORMATION *p)
{
  return p->nNumberOfLinks;
}

EIF_INTEGER posix_by_handle_file_information_nfileindexhigh(struct _BY_HANDLE_FILE_INFORMATION *p)
{
  return p->nFileIndexHigh;
}

EIF_INTEGER posix_by_handle_file_information_nfileindexlow(struct _BY_HANDLE_FILE_INFORMATION *p)
{
  return p->nFileIndexLow;
}


/* constants */

EIF_INTEGER const_duplicate_same_access()
{
  return DUPLICATE_SAME_ACCESS;
}

EIF_INTEGER const_infinite()
{
  return INFINITE;
}

EIF_INTEGER const_invalid_handle_value()
{
  return (EIF_INTEGER) (INVALID_HANDLE_VALUE);
}

EIF_POINTER const_invalid_ptr_handle_value()
{
  return (INVALID_HANDLE_VALUE);
}

EIF_INTEGER const_max_path()
{
  return (MAX_PATH);
}

EIF_INTEGER const_sw_hide()
{
  return (SW_HIDE);
}


/* constants for dwCreationFlags of CreateProcess */

EIF_INTEGER const_create_default_error_mode()
{
  return CREATE_DEFAULT_ERROR_MODE;
}

EIF_INTEGER const_create_new_console()
{
  return CREATE_NEW_CONSOLE;
}

EIF_INTEGER const_create_suspended()
{
  return CREATE_SUSPENDED;
}

EIF_INTEGER const_detached_process()
{
  return DETACHED_PROCESS;
}


/* WaitForSingleObject return values */

EIF_INTEGER const_wait_failed()
{
  return WAIT_FAILED;
}

EIF_INTEGER const_wait_abandoned	()
{
  return WAIT_ABANDONED	;
}

EIF_INTEGER const_wait_object_0()
{
  return WAIT_OBJECT_0;
}

EIF_INTEGER const_wait_timeout	()
{
  return WAIT_TIMEOUT	;
}


/* STARTUPINFO.dwFlags */

EIF_INTEGER const_startf_useshowwindow	()
{
  return STARTF_USESHOWWINDOW	;
}

EIF_INTEGER const_startf_useposition	()
{
  return STARTF_USEPOSITION	;
}

EIF_INTEGER const_startf_usesize	()
{
  return STARTF_USESIZE	;
}

EIF_INTEGER const_startf_usecountchars	()
{
  return STARTF_USECOUNTCHARS	;
}

EIF_INTEGER const_startf_usefillattribute	()
{
  return STARTF_USEFILLATTRIBUTE	;
}

EIF_INTEGER const_startf_forceonfeedback	()
{
  return STARTF_FORCEONFEEDBACK	;
}

EIF_INTEGER const_startf_forceofffeedback	()
{
  return STARTF_FORCEOFFFEEDBACK	;
}

EIF_INTEGER const_startf_usestdhandles()
{
  return STARTF_USESTDHANDLES;
}


/* GetStdHandle parameter */

EIF_INTEGER const_std_input_handle()
{
  return STD_INPUT_HANDLE;
}

EIF_INTEGER const_std_output_handle()
{
  return STD_OUTPUT_HANDLE;
}

EIF_INTEGER const_std_error_handle()
{
  return STD_ERROR_HANDLE;
}


/* constants for dwDesiredAccess of CreateFile */

EIF_INTEGER const_generic_read	()
{
  return GENERIC_READ	;
}

EIF_INTEGER const_generic_write	()
{
  return GENERIC_WRITE	;
}


/* constants for dwShareMode of CreateFile */

EIF_INTEGER const_file_share_delete	()
{
  return FILE_SHARE_DELETE	;
}

EIF_INTEGER const_file_share_read	()
{
  return FILE_SHARE_READ	;
}

EIF_INTEGER const_file_share_write	()
{
  return FILE_SHARE_WRITE	;
}


/* constants for dwCreationDistribution of CreateFile */

EIF_INTEGER const_create_new	()
{
  return CREATE_NEW	;
}

EIF_INTEGER const_create_always	()
{
  return CREATE_ALWAYS	;
}

EIF_INTEGER const_open_existing	()
{
  return OPEN_EXISTING	;
}

EIF_INTEGER const_open_always	()
{
  return OPEN_ALWAYS	;
}

EIF_INTEGER const_truncate_existing	()
{
  return TRUNCATE_EXISTING	;
}


/* constants for dwFlagsAndAttributes of CreateFile */

EIF_INTEGER const_file_attribute_archive	()
{
  return FILE_ATTRIBUTE_ARCHIVE	;
}

EIF_INTEGER const_file_attribute_compressed	()
{
  return FILE_ATTRIBUTE_COMPRESSED	;
}

EIF_INTEGER const_file_attribute_directory()
{
  return FILE_ATTRIBUTE_DIRECTORY;
}

EIF_INTEGER const_file_attribute_hidden	()
{
  return FILE_ATTRIBUTE_HIDDEN	;
}

EIF_INTEGER const_file_attribute_normal	()
{
  return FILE_ATTRIBUTE_NORMAL	;
}

EIF_INTEGER const_file_attribute_offline	()
{
  return FILE_ATTRIBUTE_OFFLINE	;
}

EIF_INTEGER const_file_attribute_readonly	()
{
  return FILE_ATTRIBUTE_READONLY	;
}

EIF_INTEGER const_file_attribute_system	()
{
  return FILE_ATTRIBUTE_SYSTEM	;
}

EIF_INTEGER const_file_attribute_temporary	()
{
  return FILE_ATTRIBUTE_TEMPORARY	;
}

EIF_INTEGER const_file_flag_write_through()
{
  return FILE_FLAG_WRITE_THROUGH;
}

EIF_INTEGER const_file_flag_overlapped()
{
  return FILE_FLAG_OVERLAPPED;
}

EIF_INTEGER const_file_flag_no_buffering()
{
  return FILE_FLAG_NO_BUFFERING;
}

EIF_INTEGER const_file_flag_random_access()
{
  return FILE_FLAG_RANDOM_ACCESS;
}

EIF_INTEGER const_file_flag_sequential_scan()
{
  return FILE_FLAG_SEQUENTIAL_SCAN;
}

EIF_INTEGER const_file_flag_delete_on_close()
{
  return FILE_FLAG_DELETE_ON_CLOSE;
}

EIF_INTEGER const_file_flag_backup_semantics()
{
  return FILE_FLAG_BACKUP_SEMANTICS;
}

EIF_INTEGER const_file_flag_posix_semantics()
{
  return FILE_FLAG_POSIX_SEMANTICS;
}


/* GetFileType constants */

EIF_INTEGER const_file_type_unknown()
{
  return FILE_TYPE_UNKNOWN;
}

EIF_INTEGER const_file_type_disk()
{
  return FILE_TYPE_DISK;
}

EIF_INTEGER const_file_type_char()
{
  return FILE_TYPE_CHAR;
}

EIF_INTEGER const_file_type_pipe()
{
  return FILE_TYPE_PIPE;
}

